//
// version helpers...
//
  GL_VERSION_1_0: Boolean = True;
  GL_VERSION_1_1: Boolean = False;
  GL_VERSION_1_2: Boolean = False;
  GL_VERSION_1_3: Boolean = False;
  GL_VERSION_1_4: Boolean = False;
  GL_VERSION_1_5: Boolean = False;
  GL_VERSION_2_0: Boolean = False;

{$IFDEF Win32}
function InitOpenGL(LibName: string = 'OpenGL32.dll'; GLULibName: string = 'GLU32.dll'): Boolean;
{$ELSE}
function InitOpenGL(LibName: string = 'libGL.so.1'; GLULibName: string = 'libGLU.so.1'): Boolean;
{$ENDIF}

implementation

{$IFDEF Win32}
uses
  Windows;
{$ENDIF}

var
  GLLibHandle: THandle = 0;
  GLULibHandle: THandle = 0;

{$IFNDEF Win32}
{$IFNDEF CLR}
const
  RTLD_LAZY             = $001;
  RTLD_NOW              = $002;
  RTLD_BINDING_MASK     = $003;      //.2
  LibraryLib = {$IFDEF Linux} 'libdl.so'{$ELSE} 'c'{$ENDIF};

function LoadLibraryEx(Name: PChar; Flags: LongInt): Pointer; cdecl; external LibraryLib name 'dlopen';
function GetProcAddressEx(Lib: Pointer; Name: PChar): Pointer; cdecl; external LibraryLib name 'dlsym';
function FreeLibraryEx(Lib: Pointer): LongInt; cdecl; external LibraryLib name 'dlclose';

function LoadLibrary(Name: PChar): THandle;
begin
  Result := THandle(LoadLibraryEx(Name, RTLD_LAZY));
end;

function GetProcAddress(LibHandle: THandle; ProcName: PChar): Pointer;
begin
  Result := GetProcAddressEx(Pointer(LibHandle), ProcName);
end;

function FreeLibrary(LibHandle: THandle): Boolean;
begin
  if LibHandle = 0 then
    Result := False
  else
    Result := FreeLibraryEx(Pointer(LibHandle)) = 0;
end;
{$ENDIF}
{$ENDIF}

function StrToInt(Str: string): Integer;
var 
  Code: Integer;
begin
  Val(Str, Result, Code);
end;

procedure ReadVersionString();
var
  vString: string;
  p, lMin, lMax: Integer;
begin
  vString := glGetString(GL_VERSION);
  p := Pos('.', vString);
  if not((p > 1) and (p < Length(vString))) then Exit;
  if not(vString[p - 1] in ['0'..'9']) then Exit;
  if not(vString[p + 1] in ['0'..'9']) then Exit;
  //
  // OK, it's a valid version string. Now remove unnecessary parts.
  //
  Dec(p);
  //
  // Find last non-numeric character before version number.
  //
  while (p > 0) and (vString[p] in ['0'..'9']) do Dec(p);
  //
  // Delete leading characters which do not belong to the version string.
  //
  Delete(vString, 1, p);
  p := Pos('.', vString) + 1;
  //
  // Find first non-numeric character after version number
  //
  while (p <= Length(vString)) and (vString[p] in ['0'..'9']) do Inc(p);
  //
  // delete trailing characters not belonging to the version string
  //
  Delete(vString, p, 255);
  //
  // Now translate the numbers.
  //
  p := Pos('.', vString); 
  lMax := StrToInt(Copy(vString, 1, p - 1));
  lMin := StrToInt(Copy(vString, p + 1, 1));
  //
  //  save settings
  //
  GL_VERSION_1_0 := True;
  GL_VERSION_1_1 := False;
  GL_VERSION_1_2 := False;
  GL_VERSION_1_3 := False;
  GL_VERSION_1_4 := False;
  GL_VERSION_1_5 := False;
  GL_VERSION_2_0 := False;

  if lMax = 1 then
  begin
    if lMin >= 1 then
    begin
      GL_VERSION_1_1 := True;
      if lMin >= 2 then GL_VERSION_1_2 := True;
      if lMin >= 3 then GL_VERSION_1_3 := True;
      if lMin >= 4 then GL_VERSION_1_4 := True;
      if lMin >= 5 then GL_VERSION_1_5 := True;
    end;
  end;

  if lMax >= 2 then
  begin
    GL_VERSION_1_1 := True;
    GL_VERSION_1_2 := True;
    GL_VERSION_1_3 := True;
    GL_VERSION_1_4 := True;
    GL_VERSION_1_5 := True;
    GL_VERSION_2_0 := True;
  end;
end;

procedure CloseLibs();
begin
  if GLLibHandle <> 0 then FreeLibrary(GLLibHandle);
  if GLULibHandle <> 0 then FreeLibrary(GLULibHandle);
end;

function InitOpenGL(LibName: string; GLULibName: string): Boolean;
begin
  Result := False;

  CloseLibs();

  GLLibHandle := LoadLibrary(PChar(LibName));
  if GLLibHandle = 0 then Exit;

  GLULibHandle := LoadLibrary(PChar(GLULibName));
  if GLULibHandle = 0 then Exit;